function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import Nivo from '../../Nivo';
import GridLines from './GridLines';
import GridLine from './GridLine';

var center = function center(scale) {
    var offset = scale.bandwidth() / 2;
    if (scale.round()) offset = Math.round(offset);

    return function (d) {
        return scale(d) + offset;
    };
};

var Grid = function (_Component) {
    _inherits(Grid, _Component);

    function Grid() {
        _classCallCheck(this, Grid);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Grid.prototype.render = function render() {
        var _props = this.props,
            width = _props.width,
            height = _props.height,
            xScale = _props.xScale,
            yScale = _props.yScale,
            theme = _props.theme,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var xLines = void 0;
        if (xScale) {
            var xValues = void 0;
            if (xScale.ticks) {
                xValues = xScale.ticks();
            } else {
                xValues = xScale.domain();
            }

            var xPosition = xScale.bandwidth ? center(xScale) : xScale;

            xLines = xValues.map(function (v) {
                return {
                    key: '' + v,
                    x1: xPosition(v),
                    x2: xPosition(v),
                    y2: height
                };
            });
        }

        var yLines = void 0;
        if (yScale) {
            var yValues = void 0;
            if (yScale.ticks) {
                yValues = yScale.ticks();
            } else {
                yValues = yScale.domain();
            }

            var yPosition = yScale.bandwidth ? center(yScale) : yScale;

            yLines = yValues.map(function (v) {
                return {
                    key: '' + v,
                    x2: width,
                    y1: yPosition(v),
                    y2: yPosition(v)
                };
            });
        }

        return React.createElement(
            'g',
            null,
            xLines && React.createElement(GridLines, {
                type: 'x',
                lines: xLines,
                theme: theme,
                animate: animate,
                motionStiffness: motionStiffness,
                motionDamping: motionDamping
            }),
            yLines && React.createElement(GridLines, {
                type: 'y',
                lines: yLines,
                theme: theme,
                animate: animate,
                motionStiffness: motionStiffness,
                motionDamping: motionDamping
            })
        );
    };

    return Grid;
}(Component);

Grid.propTypes = {
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,

    xScale: PropTypes.func,
    yScale: PropTypes.func,

    theme: PropTypes.object.isRequired,

    // motion
    animate: PropTypes.bool.isRequired,
    motionStiffness: PropTypes.number.isRequired,
    motionDamping: PropTypes.number.isRequired
};
Grid.defaultProps = {
    // motion
    animate: true,
    motionStiffness: Nivo.defaults.motionStiffness,
    motionDamping: Nivo.defaults.motionDamping
};
export default Grid;