var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { TransitionMotion, spring } from 'react-motion';
import { motion as motionPropTypes } from '../../../PropTypes';

var LineMarkers = function (_Component) {
    _inherits(LineMarkers, _Component);

    function LineMarkers() {
        _classCallCheck(this, LineMarkers);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    LineMarkers.prototype.render = function render() {
        var _props = this.props,
            lines = _props.lines,
            size = _props.size,
            color = _props.color,
            borderWidth = _props.borderWidth,
            borderColor = _props.borderColor,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var points = lines.reduce(function (acc, line) {
            var id = line.id,
                points = line.points;


            return [].concat(acc, points.map(function (point) {
                return {
                    key: id + '.' + point.x,
                    x: point.x,
                    y: point.y,
                    fill: color(line),
                    stroke: borderColor(line)
                };
            }));
        }, []);

        var circles = void 0;
        if (animate === true) {
            var springConfig = {
                motionDamping: motionDamping,
                motionStiffness: motionStiffness
            };

            circles = React.createElement(
                TransitionMotion,
                {
                    styles: points.map(function (point) {
                        return {
                            key: point.key,
                            data: {
                                fill: point.fill,
                                stroke: point.stroke
                            },
                            style: {
                                x: spring(point.x, springConfig),
                                y: spring(point.y, springConfig),
                                size: spring(size, springConfig)
                            }
                        };
                    })
                },
                function (interpolatedStyles) {
                    return React.createElement(
                        'g',
                        null,
                        interpolatedStyles.map(function (_ref) {
                            var key = _ref.key,
                                style = _ref.style,
                                data = _ref.data;
                            return React.createElement('circle', {
                                key: key,
                                cx: style.x,
                                cy: style.y,
                                r: style.size / 2,
                                fill: data.fill,
                                stroke: data.stroke,
                                strokeWidth: borderWidth
                            });
                        })
                    );
                }
            );
        } else {
            circles = points.map(function (point) {
                return React.createElement('circle', {
                    key: point.key,
                    cx: point.x,
                    cy: point.y,
                    r: size / 2,
                    fill: point.fill,
                    stroke: point.stroke,
                    strokeWidth: borderWidth
                });
            });
        }

        return React.createElement(
            'g',
            null,
            circles
        );
    };

    return LineMarkers;
}(Component);

LineMarkers.propTypes = _extends({
    lines: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string.isRequired
    })),
    size: PropTypes.number.isRequired,
    color: PropTypes.func.isRequired,
    borderWidth: PropTypes.number.isRequired,
    borderColor: PropTypes.func.isRequired
}, motionPropTypes);
LineMarkers.defaultProps = {
    animate: true,
    motionStiffness: 90,
    motionDamping: 15
};
export default LineMarkers;