var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { merge } from 'lodash-es';
import { Motion, TransitionMotion, spring } from 'react-motion';
import Nivo, { defaultTheme } from '../../../Nivo';
import { margin as marginPropType, motion as motionPropTypes } from '../../../PropTypes';
import { getColorsGenerator, getInheritedColorGenerator } from '../../../ColorUtils';
import { getLabelGenerator } from '../../../lib/propertiesConverters';
import { degreesToRadians } from '../../../ArcUtils';
import SvgWrapper from '../SvgWrapper';
import { pie as d3Pie, arc as d3Arc } from 'd3-shape';
import PieRadialLabels from './PieRadialLabels';
import PieSlicesLabels from './PieSlicesLabels';

var Pie = function (_Component) {
    _inherits(Pie, _Component);

    function Pie() {
        _classCallCheck(this, Pie);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Pie.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            _margin = _props.margin,
            _width = _props.width,
            _height = _props.height,
            _innerRadius = _props.innerRadius,
            _padAngle = _props.padAngle,
            cornerRadius = _props.cornerRadius,
            borderWidth = _props.borderWidth,
            _borderColor = _props.borderColor,
            enableRadialLabels = _props.enableRadialLabels,
            radialLabel = _props.radialLabel,
            radialLabelsLinkOffset = _props.radialLabelsLinkOffset,
            radialLabelsLinkDiagonalLength = _props.radialLabelsLinkDiagonalLength,
            radialLabelsLinkHorizontalLength = _props.radialLabelsLinkHorizontalLength,
            radialLabelsLinkStrokeWidth = _props.radialLabelsLinkStrokeWidth,
            radialLabelsTextXOffset = _props.radialLabelsTextXOffset,
            radialLabelsTextColor = _props.radialLabelsTextColor,
            radialLabelsLinkColor = _props.radialLabelsLinkColor,
            enableSlicesLabels = _props.enableSlicesLabels,
            sliceLabel = _props.sliceLabel,
            slicesLabelsTextColor = _props.slicesLabelsTextColor,
            _theme = _props.theme,
            colors = _props.colors,
            colorBy = _props.colorBy,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var margin = Object.assign({}, Nivo.defaults.margin, _margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;
        var centerX = width / 2;
        var centerY = height / 2;

        var padAngle = degreesToRadians(_padAngle);

        var theme = merge({}, defaultTheme, _theme);
        var color = getColorsGenerator(colors, colorBy);
        var borderColor = getInheritedColorGenerator(_borderColor);

        var motionProps = {
            animate: animate,
            motionDamping: motionDamping,
            motionStiffness: motionStiffness
        };

        var radialLabelsProps = {
            label: getLabelGenerator(radialLabel),
            linkOffset: radialLabelsLinkOffset,
            linkDiagonalLength: radialLabelsLinkDiagonalLength,
            linkHorizontalLength: radialLabelsLinkHorizontalLength,
            linkStrokeWidth: radialLabelsLinkStrokeWidth,
            textXOffset: radialLabelsTextXOffset,
            textColor: getInheritedColorGenerator(radialLabelsTextColor, 'axis.textColor'),
            linkColor: getInheritedColorGenerator(radialLabelsLinkColor, 'axis.tickColor')
        };

        var slicesLabelsProps = {
            label: getLabelGenerator(sliceLabel),
            textColor: getInheritedColorGenerator(slicesLabelsTextColor, 'axis.textColor')
        };

        var radius = Math.min(width, height) / 2;
        var innerRadius = radius * Math.min(_innerRadius, 1);

        var pie = d3Pie();
        pie.value(function (d) {
            return d.value;
        });

        var arc = d3Arc();
        arc.outerRadius(radius);

        return React.createElement(
            SvgWrapper,
            { width: _width, height: _height, margin: margin },
            React.createElement(
                Motion,
                {
                    style: {
                        centerX: spring(centerX, motionProps),
                        centerY: spring(centerY, motionProps),
                        innerRadius: spring(innerRadius),
                        padAngle: spring(padAngle, motionProps),
                        cornerRadius: spring(cornerRadius, motionProps)
                    }
                },
                function (interpolatingStyle) {
                    var interpolatedPie = pie.padAngle(interpolatingStyle.padAngle);
                    var interpolatedArc = arc.cornerRadius(interpolatingStyle.cornerRadius).innerRadius(interpolatingStyle.innerRadius);

                    var arcsData = interpolatedPie(data).map(function (d) {
                        return _extends({}, d, {
                            data: _extends({}, d.data, {
                                color: color(d.data)
                            })
                        });
                    });

                    return React.createElement(
                        'g',
                        {
                            transform: 'translate(' + interpolatingStyle.centerX + ', ' + interpolatingStyle.centerY + ')'
                        },
                        arcsData.map(function (d) {
                            return React.createElement('path', {
                                key: d.data.id,
                                d: interpolatedArc(d),
                                fill: d.data.color,
                                strokeWidth: borderWidth,
                                stroke: borderColor(d.data)
                            });
                        }),
                        enableSlicesLabels && React.createElement(PieSlicesLabels, _extends({
                            data: arcsData,
                            radius: radius,
                            innerRadius: interpolatingStyle.innerRadius,
                            theme: theme
                        }, slicesLabelsProps)),
                        enableRadialLabels && React.createElement(PieRadialLabels, _extends({
                            data: arcsData,
                            radius: radius,
                            theme: theme
                        }, radialLabelsProps))
                    );
                }
            )
        );
    };

    return Pie;
}(Component);

Pie.propTypes = _extends({
    data: PropTypes.arrayOf(PropTypes.shape({
        id: PropTypes.string.isRequired,
        value: PropTypes.number.isRequired
    })).isRequired,

    // dimensions
    width: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    margin: marginPropType,

    innerRadius: PropTypes.number.isRequired,
    padAngle: PropTypes.number.isRequired,
    cornerRadius: PropTypes.number.isRequired,

    // border
    borderWidth: PropTypes.number.isRequired,
    borderColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),

    // radial labels
    enableRadialLabels: PropTypes.bool.isRequired,
    radialLabel: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
    radialLabelsTextXOffset: PropTypes.number,
    radialLabelsTextColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
    radialLabelsLinkOffset: PropTypes.number,
    radialLabelsLinkDiagonalLength: PropTypes.number,
    radialLabelsLinkHorizontalLength: PropTypes.number,
    radialLabelsLinkStrokeWidth: PropTypes.number,
    radialLabelsLinkColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),

    // slices labels
    enableSlicesLabels: PropTypes.bool.isRequired,
    sliceLabel: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
    slicesLabelsTextColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),

    // theming
    theme: PropTypes.object.isRequired,
    colors: PropTypes.any.isRequired,
    colorBy: PropTypes.oneOfType([PropTypes.string, PropTypes.func])

}, motionPropTypes);
Pie.defaultProps = {
    // dimensions
    margin: Nivo.defaults.margin,

    innerRadius: 0,
    padAngle: 0,
    cornerRadius: 0,

    // border
    borderWidth: 0,
    borderColor: 'inherit:darker(1)',

    // radial labels
    enableRadialLabels: true,
    radialLabel: 'id',
    radialLabelsTextColor: 'theme',
    radialLabelsLinkColor: 'theme',

    // slices labels
    enableSlicesLabels: true,
    sliceLabel: 'value',
    slicesLabelsTextColor: 'theme',

    // theming
    theme: {},
    colors: Nivo.defaults.colorRange,
    colorBy: 'id',

    // motion
    animate: true,
    motionStiffness: Nivo.defaults.motionStiffness,
    motionDamping: Nivo.defaults.motionDamping
};
export default Pie;