function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { findDOMNode } from 'react-dom';
import d3 from 'd3';
import _ from 'lodash-es';
import Nivo from '../../../Nivo';
import { lineInterpolation } from '../../../PropTypes';
import { getColorRange, getColorGenerator } from '../../../ColorUtils';
import { margin as marginPropType } from '../../../PropTypes';
import decoratorsFromReactChildren from '../../../lib/decoratorsFromReactChildren';

var findPrecedingLayer = function findPrecedingLayer(layers, index) {
    if (layers === null) {
        return null;
    }

    for (var i = layers.length - 1; i >= 0; i--) {
        var layer = layers[i];
        if (layer.index < index) {
            return layer;
        }
    }

    return null;
};

var Stack = function (_Component) {
    _inherits(Stack, _Component);

    function Stack(props) {
        _classCallCheck(this, Stack);

        var _this = _possibleConstructorReturn(this, _Component.call(this, props));

        _this.state = {
            excludeLayers: []
        };

        _this.previousData = null;
        return _this;
    }

    Stack.prototype.renderD3 = function renderD3(props, state) {
        var _this2 = this;

        var data = props.data,
            offset = props.offset,
            interpolation = props.interpolation,
            colors = props.colors,
            overColor = props.overColor,
            transitionDuration = props.transitionDuration,
            transitionEasing = props.transitionEasing;
        var excludeLayers = state.excludeLayers;


        var overColorFn = getColorGenerator(overColor);

        var element = d3.select(findDOMNode(this));
        var wrapper = element.select('.nivo_stack_wrapper');

        var margin = _.assign({}, Nivo.defaults.margin, props.margin);
        var width = props.width - margin.left - margin.right;
        var height = props.height - margin.top - margin.bottom;

        element.attr({
            width: props.width,
            height: props.height
        });
        wrapper.attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');

        var color = getColorRange(colors);

        var normData = _.cloneDeep(data).map(function (values, i) {
            return {
                index: i,
                color: color(i),
                values: values
            };
        });

        var filteredData = [];
        var hiddenData = [];
        if (excludeLayers.length > 0) {
            normData.forEach(function (layer) {
                if (excludeLayers.indexOf(layer.index) === -1) {
                    filteredData.push(layer);
                } else {
                    hiddenData.push(layer);
                }
            });
        } else {
            filteredData = normData;
        }

        var stack = d3.layout.stack().offset(offset).values(function (d) {
            return d.values;
        });

        var stacked = stack(filteredData);

        var xScale = d3.scale.linear().range([0, width]).domain([0, stacked[0].values.length - 1]);

        var yScale = d3.scale.linear().range([height, 0]).domain([0, d3.max(stacked, function (layer) {
            return d3.max(layer.values, function (d) {
                return d.y0 + d.y;
            });
        })]);

        filteredData = filteredData.map(function (layer) {
            return _.assign(layer, {
                values: layer.values.map(function (v) {
                    return {
                        value: v,
                        interpolated: {
                            x: xScale(v.x),
                            y0: yScale(v.y0),
                            y: yScale(v.y0 + v.y)
                        }
                    };
                })
            });
        });

        var area = d3.svg.area().interpolate(interpolation).x(function (d) {
            return d.x;
        }).y0(function (d) {
            return d.y0;
        }).y1(function (d) {
            return d.y;
        });

        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        // Areas
        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        var paths = wrapper.select('.nivo_stack_areas').selectAll('.nivo_stack_area').data(stacked, function (d) {
            return d.index;
        });

        // ENTER
        paths.enter().append('path').attr('class', 'nivo_stack_area').attr('d', function (d) {
            if (_this2.previousData === null) {
                return area(d.values.map(function (p) {
                    return {
                        x: p.interpolated.x,
                        y0: yScale.range()[0],
                        y: yScale.range()[0]
                    };
                }));
            }

            var precedingLayer = findPrecedingLayer(_this2.previousData, d.index);

            if (precedingLayer !== null) {
                return area(precedingLayer.values.map(function (p) {
                    return {
                        x: p.interpolated.x,
                        y0: p.interpolated.y,
                        y: p.interpolated.y
                    };
                }));
            }

            return area(d.values.map(function (p) {
                return {
                    x: p.interpolated.x,
                    y0: p.interpolated.y0,
                    y: p.interpolated.y0
                };
            }));
        }).style('fill', function (d) {
            return d.color;
        });

        // UPDATE
        paths.on('click', function (d) {
            // we cannot have no layer
            if (filteredData.length > 1) {
                _this2.setState({
                    excludeLayers: excludeLayers.concat([d.index])
                });
            }
        }).on('mouseover', function (d) {
            d3.select(this).style('fill', overColorFn);
        }).on('mousemove', function (d, i) {
            d3.select(this).style('fill', overColorFn);
        }).on('mouseout', function (d) {
            d3.select(this).style('fill', d.color);
        }).transition().duration(transitionDuration).ease(transitionEasing).attr('d', function (d) {
            return area(d.values.map(function (v) {
                return v.interpolated;
            }));
        }).style('fill', function (d) {
            return d.color;
        });

        // EXIT
        paths.exit().transition().duration(transitionDuration).ease(transitionEasing).attr('d', function (d) {
            var precedingLayer = findPrecedingLayer(filteredData, d.index);

            if (precedingLayer !== null) {
                return area(precedingLayer.values.map(function (p) {
                    return {
                        x: p.interpolated.x,
                        y0: p.interpolated.y,
                        y: p.interpolated.y
                    };
                }));
            }

            return area(d.values.map(function (p) {
                return {
                    x: p.interpolated.x,
                    y0: p.interpolated.y0,
                    y: p.interpolated.y0
                };
            }));
        }).remove();

        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        // Hidden layers
        // —————————————————————————————————————————————————————————————————————————————————————————————————————————————
        var hiddenControls = wrapper.selectAll('.nivo_stack_hidden_control').data(hiddenData, function (d) {
            return d.index;
        });

        // ENTER
        hiddenControls.enter().append('circle').attr('class', 'nivo_stack_hidden_control').attr('r', 8).attr('transform', function (d, i) {
            return 'translate(' + i * 40 + ',0)';
        }).style('fill', function (d) {
            return d.color;
        }).style('opacity', 0).style('cursor', 'pointer');

        hiddenControls.on('click', function (d) {
            _this2.setState({
                excludeLayers: excludeLayers.filter(function (index) {
                    return index !== d.index;
                })
            });
        }).transition().duration(transitionDuration).ease(transitionEasing).attr('transform', function (d, i) {
            return 'translate(' + i * 40 + ',0)';
        }).style('fill', function (d) {
            return d.color;
        }).style('opacity', 1);

        hiddenControls.exit().transition().duration(transitionDuration).ease(transitionEasing).style('opacity', 0).remove();

        var stackContext = {
            element: element,
            wrapper: wrapper,
            width: width,
            height: height,
            stacked: stacked,
            xScale: xScale,
            yScale: yScale,
            transitionDuration: transitionDuration,
            transitionEasing: transitionEasing
        };

        this.decorators.forEach(function (decorator) {
            decorator(stackContext);
        });

        this.previousData = filteredData;
    };

    Stack.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
        this.decorators = decoratorsFromReactChildren(nextProps.children, 'decorateStack');

        this.renderD3(nextProps, nextState);

        return false;
    };

    Stack.prototype.componentDidMount = function componentDidMount() {
        this.decorators = decoratorsFromReactChildren(this.props.children, 'decorateStack');

        this.renderD3(this.props, this.state);
    };

    Stack.prototype.render = function render() {
        return React.createElement(
            'svg',
            { ref: 'svg', className: 'nivo_stack' },
            React.createElement(
                'g',
                { className: 'nivo_stack_wrapper' },
                React.createElement('g', { className: 'nivo_stack_areas' })
            )
        );
    };

    return Stack;
}(Component);

var array = PropTypes.array,
    number = PropTypes.number,
    string = PropTypes.string,
    func = PropTypes.func,
    any = PropTypes.any,
    oneOf = PropTypes.oneOf;


Stack.propTypes = {
    width: number.isRequired,
    height: number.isRequired,
    margin: marginPropType,
    sort: func,
    data: array.isRequired,
    offset: oneOf(['silhouette', 'wiggle', 'expand', 'zero']).isRequired,
    keyProp: string.isRequired,
    valueProp: string.isRequired,
    interpolation: lineInterpolation,
    colors: any.isRequired,
    overColor: any.isRequired,
    transitionDuration: number.isRequired,
    transitionEasing: string.isRequired
};

Stack.defaultProps = {
    margin: Nivo.defaults.margin,
    sort: null,
    offset: 'zero',
    keyProp: 'label',
    valueProp: 'value',
    interpolation: 'monotone',
    transitionDuration: Nivo.defaults.transitionDuration,
    transitionEasing: Nivo.defaults.transitionEasing,
    colors: Nivo.defaults.colorRange,
    overColor: 'inherit:brighter(.5)'
};

export default Stack;