'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactMotion = require('react-motion');

var _PropTypes = require('../../PropTypes');

var _Nivo = require('../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _AxisTick = require('./AxisTick');

var _AxisTick2 = _interopRequireDefault(_AxisTick);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var center = function center(scale) {
    var offset = scale.bandwidth() / 2;
    if (scale.round()) {
        offset = Math.round(offset);
    }

    return function (d) {
        return scale(d) + offset;
    };
};

var Axis = function (_Component) {
    _inherits(Axis, _Component);

    function Axis() {
        _classCallCheck(this, Axis);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Axis.prototype.willEnter = function willEnter() {
        return {
            opacity: 0,
            x: 0,
            y: 0
        };
    };

    Axis.prototype.willLeave = function willLeave(_ref) {
        var style = _ref.style;

        return {
            opacity: (0, _reactMotion.spring)(0),
            x: (0, _reactMotion.spring)(style.x.val),
            y: (0, _reactMotion.spring)(style.y.val)
        };
    };

    Axis.prototype.render = function render() {
        var _props = this.props,
            scale = _props.scale,
            width = _props.width,
            height = _props.height,
            _position = _props.position,
            _orient = _props.orient,
            tickSize = _props.tickSize,
            tickPadding = _props.tickPadding,
            format = _props.format,
            _legend = _props.legend,
            legendPosition = _props.legendPosition,
            legendOffset = _props.legendOffset,
            theme = _props.theme,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var values = void 0;
        if (scale.ticks) {
            values = scale.ticks();
        } else {
            values = scale.domain();
        }

        var orient = _orient || _position;
        var position = scale.bandwidth ? center(scale) : scale;

        var x = 0;
        var y = 0;
        var translate = void 0;
        var textAnchor = void 0;
        var textDY = void 0;
        var tickLine = { x2: 0, y2: 0 };
        var textXY = { x: 0, y: 0 };

        if (['top', 'bottom'].includes(orient)) {
            translate = function translate(d) {
                return { x: position(d), y: 0 };
            };

            textAnchor = 'middle';
            textDY = orient === 'top' ? '0em' : '0.71em';
            textXY.y = (tickSize + tickPadding) * (orient === 'bottom' ? 1 : -1);

            tickLine.y2 = tickSize * (orient === 'bottom' ? 1 : -1);

            if (orient === 'bottom') y = height;
        } else if (['left', 'right'].includes(orient)) {
            translate = function translate(d) {
                return { x: 0, y: position(d) };
            };

            textAnchor = orient === 'left' ? 'end' : 'start';
            textDY = '0.32em';
            textXY.x = (tickSize + tickPadding) * (orient === 'right' ? 1 : -1);

            tickLine.x2 = tickSize * (orient === 'right' ? 1 : -1);

            if (orient === 'right') x = width;
        }

        var legend = null;
        if (_legend !== undefined) {
            var legendX = 0;
            var legendY = 0;
            var legendRotation = 0;
            var _textAnchor = void 0;

            if (['left', 'right'].includes(_position)) {
                legendRotation = -90;
                legendX = legendOffset;
                if (legendPosition === 'start') {
                    _textAnchor = 'start';
                    legendY = height;
                } else if (legendPosition === 'center') {
                    _textAnchor = 'middle';
                    legendY = height / 2;
                } else if (legendPosition === 'end') {
                    _textAnchor = 'end';
                }
            } else {
                legendY = legendOffset;
                if (legendPosition === 'start') {
                    _textAnchor = 'start';
                } else if (legendPosition === 'center') {
                    _textAnchor = 'middle';
                    legendX = width / 2;
                } else if (legendPosition === 'end') {
                    _textAnchor = 'end';
                    legendX = width;
                }
            }

            legend = _react2.default.createElement(
                'text',
                {
                    fill: theme.axis.legendColor,
                    transform: 'translate(' + legendX + ', ' + legendY + ') rotate(' + legendRotation + ')',
                    textAnchor: _textAnchor,
                    style: { fontSize: theme.axis.legendFontSize }
                },
                _legend
            );
        }

        var ticks = values.map(function (v) {
            return _extends({
                key: v
            }, translate(v));
        });

        var tickElements = void 0;
        if (!animate) {
            tickElements = _react2.default.createElement(
                'g',
                null,
                ticks.map(function (tick) {
                    return _react2.default.createElement(_AxisTick2.default, {
                        key: tick.key,
                        value: tick.key,
                        format: format,
                        tickLine: tickLine,
                        textXY: textXY,
                        textDY: textDY,
                        textAnchor: textAnchor,
                        theme: theme,
                        x: tick.x,
                        y: tick.y
                    });
                })
            );
        } else {
            var springConfig = {
                stiffness: motionStiffness,
                damping: motionDamping
            };

            tickElements = _react2.default.createElement(
                _reactMotion.TransitionMotion,
                {
                    willEnter: this.willEnter,
                    willLeave: this.willLeave,
                    styles: ticks.map(function (tick) {
                        return {
                            key: '' + tick.key,
                            data: tick.key,
                            style: {
                                opacity: (0, _reactMotion.spring)(1, springConfig),
                                x: (0, _reactMotion.spring)(tick.x, springConfig),
                                y: (0, _reactMotion.spring)(tick.y, springConfig)
                            }
                        };
                    })
                },
                function (interpolatedStyles) {
                    return _react2.default.createElement(
                        'g',
                        null,
                        interpolatedStyles.map(function (_ref2) {
                            var key = _ref2.key,
                                style = _ref2.style;
                            return _react2.default.createElement(_AxisTick2.default, _extends({
                                key: key,
                                value: key,
                                format: format,
                                tickLine: tickLine,
                                textXY: textXY,
                                textDY: textDY,
                                textAnchor: textAnchor,
                                theme: theme
                            }, style));
                        })
                    );
                }
            );
        }

        return _react2.default.createElement(
            'g',
            {
                className: 'nivo__axis nivo__axis--' + _position + '  nivo__axis--orient-' + orient,
                transform: 'translate(' + x + ',' + y + ')'
            },
            legend,
            tickElements
        );
    };

    return Axis;
}(_react.Component);

Axis.propTypes = _extends({
    width: _propTypes2.default.number.isRequired,
    height: _propTypes2.default.number.isRequired,
    orient: _propTypes2.default.oneOf(['top', 'right', 'bottom', 'left']),
    position: _propTypes2.default.oneOf(['top', 'right', 'bottom', 'left']).isRequired,
    scale: _propTypes2.default.func.isRequired,

    // ticks
    tickSize: _propTypes2.default.number.isRequired,
    tickPadding: _propTypes2.default.number.isRequired,
    format: _propTypes2.default.func,

    // legend
    legend: _propTypes2.default.string,
    legendPosition: _propTypes2.default.oneOf(['start', 'center', 'end']).isRequired,
    legendOffset: _propTypes2.default.number.isRequired,
    theme: _propTypes2.default.object.isRequired

}, _PropTypes.motion);
Axis.defaultProps = {
    // ticks
    tickSize: 5,
    tickPadding: 5,

    // legend
    legendPosition: 'end',
    legendOffset: 0,

    // motion
    animate: true,
    motionStiffness: _Nivo2.default.defaults.motionStiffness,
    motionDamping: _Nivo2.default.defaults.motionDamping
};
exports.default = Axis;