'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _d3Shape = require('d3-shape');

var _Nivo = require('../../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _PropTypes = require('../../../PropTypes');

var _ColorUtils = require('../../../ColorUtils');

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _line = require('../../../lib/charts/line');

var _curve = require('../../../properties/curve');

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _LineMarkers = require('./LineMarkers');

var _LineMarkers2 = _interopRequireDefault(_LineMarkers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * This file is part of the nivo project.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * file that was distributed with this source code.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var Line = function (_Component) {
    _inherits(Line, _Component);

    function Line() {
        _classCallCheck(this, Line);

        return _possibleConstructorReturn(this, _Component.apply(this, arguments));
    }

    Line.prototype.render = function render() {
        var _props = this.props,
            data = _props.data,
            stacked = _props.stacked,
            curve = _props.curve,
            _margin = _props.margin,
            _width = _props.width,
            _height = _props.height,
            axes = _props.axes,
            enableGridX = _props.enableGridX,
            enableGridY = _props.enableGridY,
            enableMarkers = _props.enableMarkers,
            markersSize = _props.markersSize,
            markersColor = _props.markersColor,
            markersBorderWidth = _props.markersBorderWidth,
            markersBorderColor = _props.markersBorderColor,
            _theme = _props.theme,
            colors = _props.colors,
            colorBy = _props.colorBy,
            animate = _props.animate,
            motionStiffness = _props.motionStiffness,
            motionDamping = _props.motionDamping;


        var margin = Object.assign({}, _Nivo2.default.defaults.margin, _margin);
        var width = _width - margin.left - margin.right;
        var height = _height - margin.top - margin.bottom;

        var theme = (0, _lodash.merge)({}, _Nivo.defaultTheme, _theme);
        var color = (0, _ColorUtils.getColorsGenerator)(colors, colorBy);

        var motionProps = {
            animate: animate,
            motionDamping: motionDamping,
            motionStiffness: motionStiffness
        };

        var result = void 0;
        if (stacked === true) {
            result = (0, _line.generateStackedLines)(data, width, height, color);
        } else {
            result = (0, _line.generateLines)(data, width, height, color);
        }

        var lineGenerator = (0, _d3Shape.line)().x(function (d) {
            return d.x;
        }).y(function (d) {
            return d.y;
        }).curve(_curve.curvePropMapping[curve]);

        var _result = result,
            xScale = _result.xScale,
            yScale = _result.yScale,
            lines = _result.lines;


        var markers = null;
        if (enableMarkers === true) {
            markers = _react2.default.createElement(_LineMarkers2.default, _extends({
                lines: lines,
                size: markersSize,
                color: (0, _ColorUtils.getColorGenerator)(markersColor),
                borderWidth: markersBorderWidth,
                borderColor: (0, _ColorUtils.getColorGenerator)(markersBorderColor)
            }, motionProps));
        }

        return _react2.default.createElement(
            _SvgWrapper2.default,
            { width: _width, height: _height, margin: margin },
            _react2.default.createElement(_Grid2.default, _extends({
                theme: theme,
                width: width,
                height: height,
                xScale: enableGridX ? xScale : null,
                yScale: enableGridY ? yScale : null
            }, motionProps)),
            _react2.default.createElement(_Axes2.default, _extends({
                axes: axes,
                xScale: xScale,
                yScale: yScale,
                width: width,
                height: height,
                theme: theme
            }, motionProps)),
            lines.map(function (_ref) {
                var id = _ref.id,
                    lineColor = _ref.color,
                    points = _ref.points;
                return _react2.default.createElement('path', {
                    key: id,
                    d: lineGenerator(points),
                    fill: 'none',
                    strokeWidth: 2,
                    stroke: lineColor
                });
            }),
            markers
        );
    };

    return Line;
}(_react.Component);

Line.propTypes = _extends({
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
        id: _propTypes2.default.string.isRequired,
        data: _propTypes2.default.arrayOf(_propTypes2.default.shape({
            x: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]).isRequired,
            y: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]).isRequired
        })).isRequired
    })).isRequired,

    stacked: _propTypes2.default.bool.isRequired,
    curve: _curve.curvePropType.isRequired,

    // dimensions
    width: _propTypes2.default.number.isRequired,
    height: _propTypes2.default.number.isRequired,
    margin: _PropTypes.margin,

    // axes
    axes: _propTypes2.default.object.isRequired,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // markers
    enableMarkers: _propTypes2.default.bool.isRequired,
    markersSize: _propTypes2.default.number.isRequired,
    markersColor: _propTypes2.default.any.isRequired,
    markersBorderWidth: _propTypes2.default.number.isRequired,
    markersBorderColor: _propTypes2.default.any.isRequired,

    // theming
    theme: _propTypes2.default.object.isRequired,
    colors: _propTypes2.default.any.isRequired,
    colorBy: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func])

}, _PropTypes.motion);
Line.defaultProps = {
    stacked: false,
    curve: 'linear',

    // dimensions
    margin: _Nivo2.default.defaults.margin,

    // axes
    axes: {
        left: {
            enabled: true
        },
        bottom: {
            enabled: true
        }
    },
    enableGridX: true,
    enableGridY: true,

    // markers
    enableMarkers: true,
    markersSize: 6,
    markersColor: 'inherit',
    markersBorderWidth: 0,
    markersBorderColor: 'inherit',

    // theming
    theme: {},
    colors: _Nivo2.default.defaults.colorRange,
    colorBy: 'id',

    // motion
    animate: true,
    motionStiffness: _Nivo2.default.defaults.motionStiffness,
    motionDamping: _Nivo2.default.defaults.motionDamping
};
exports.default = Line;